const reducers = require('../reducers');

describe('Reducers reduce two values to one.', () => {
  describe('allEqual()', () => {
    test('equality returns the value', () => {
      const result = reducers.allEqual(1, 1);
      expect(result).toBe(1);
    });
    test('inequality returns  "Mixed"', () => {
      const result = reducers.allEqual(1, 2);
      expect(result).toBe('Mixed');
    });
  });

  describe('allTrue()', () => {
    test('true && true => true', () => {
      const result = reducers.allTrue(true, true);
      expect(result).toBe(true);
    });
    test('true && false => false', () => {
      const result = reducers.allTrue(true, false);
      expect(result).toBe(false);
    });
    test('false && true => false', () => {
      const result = reducers.allTrue(false, true);
      expect(result).toBe(false);
    });
    test('false && false => false', () => {
      const result = reducers.allTrue(false, false);
      expect(result).toBe(false);
    });
  });

  describe('blank()', () => {
    test('it returns empty string', () => {
      const result = reducers.blank();
      expect(result).toBe('');
    });
  });

  describe('max()', () => {
    test('bigger, smaller => bigger', () => {
      const result = reducers.max(1, 0);
      expect(result).toBe(1);
    });
    test('smaller, bigger => bigger', () => {
      const result = reducers.max(0, 1);
      expect(result).toBe(1);
    });
    test('same, same => same', () => {
      const result = reducers.max(1, 1);
      expect(result).toBe(1);
    });
  });

  describe('mean()', () => {
    test('it works for integers', () => {
      const result = [0, 1, 2].reduce(reducers.mean);
      expect(result).toBe(1);
    });
    test('it works for floats', () => {
      const array = [0, 1];
      const result = array.reduce(reducers.mean);
      expect(result).toBe(0.5);
    });
  });

  describe('sum', () => {
    test('it all adds up', () => {
      const result = [0, 1, 2].reduce(reducers.sum);
      expect(result).toBe(3);
    });
  });
});
