const path = require('path');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');

module.exports = (env) => {
  // make js loader based on platform
  const jsLoaders = [];
  if (env.platform === 'web') {
    jsLoaders.push({
      loader: 'babel-loader',
      options: { presets: ['@babel/preset-env'] },
    });
  }
  const entryPoint = './src/PivotTableCore.js';
  const buildDir = `dist/${env.NODE_ENV}/${env.platform}`;
  const devtool = (env.NODE_ENV === 'development') ? 'source-map' : false;

  const output = {
    path: path.resolve(__dirname, buildDir),
    filename: '[name].js',
  };

  if (env.platform === 'web') {
    output.library = 'PivotTableCore';
  }

  return {
    entry: {
      PivotTableCore: entryPoint,
      'PivotTableCore.min': entryPoint,
    },
    output,
    devtool,
    mode: env.NODE_ENV,
    target: env.platform,
    module: {
      rules: [
        {
          test: /.js$/,
          exclude: /node_modules/,
          use: jsLoaders,
        },
      ],
    },
    plugins: [
      new CleanWebpackPlugin([buildDir]),
    ],
    optimization: {
      minimize: (env.NODE_ENV === 'production'),
      minimizer: [new UglifyJsPlugin({ include: /\.min\.js$/ })],
    },
  };
};
